import React from 'react';
import ReactDOM from 'react-dom';
import cx from "classnames";

import { connect } from "react-redux";
import { handleModalSelectChange, handleStartDateChange, handleStartTimeChange,
  handleEndDateChange, handleEndTimeChange, handleBookingSave,
  handleBookingDelete, closeModal
} from "../../redux/advanced/actions";

class AdvancedBookingForm extends React.Component {
  // Delegate the actions to the reducers.
  handleModalSelectChange = (e) => {
    this.props.handleModalSelectChange(e);
  }
  handleStartDateChange = (e) => {
    this.props.handleStartDateChange(e);
  };
  handleStartTimeChange = (e) => {
    this.props.handleStartTimeChange(e);
  };
  handleEndDateChange = (e) => {
    this.props.handleEndDateChange(e);
  };
  handleEndTimeChange = (e) => {
    this.props.handleEndTimeChange(e);
  };
  handleBookingSave = () => {
    this.props.handleBookingSave(this.props.settings, this.props.modal, this.props.oldestBooking);
  };
  handleBookingDelete = () => {
    this.props.handleBookingDelete(this.props.settings, this.props.modal);
  };

  render() {
    return (
      <div>
        <form className="advanced-booking-form">
          <div className="form-group">
            <label htmlFor="advanced-booking-form-state-select">State</label>
            <select
              className="form-control"
              id="advanced-booking-form-state-select"
              value={this.props.modal.modalStateSelect}
              onChange={this.handleModalSelectChange}>
              {Object.keys(this.props.settings.dataStates).map(key => {
                return <option key={key} value={key}>{this.props.settings.dataStates[key]}</option>;
              })};
            </select>
          </div>
          <div className='row'>
            <div className="form-group col">
              <label htmlFor="advanced-booking-form-state-period-start-date">From</label>
              <input
                className="form-control"
                id="advanced-booking-form-state-period-start-date"
                type="date"
                placeholder="YYYY-mm-dd"
                required pattern="[0-9]{2}.[0-9]{2}.[0-9]{4}"
                value={this.props.modal.modalStartDate}
                onChange={this.handleStartDateChange}
              />
              <input
                className="form-control"
                id="advanced-booking-form-state-period-start-time"
                type="time"
                placeholder="HH:mm"
                required pattern="[0-9]{2}:[0-9]{2}"
                value={this.props.modal.modalStartTime}
                onChange={this.handleStartTimeChange}
              />
            </div>
            <div className="form-group col">
              <label htmlFor="advanced-booking-form-state-period-ebd-date">To</label>
              <input
                className="form-control"
                id="advanced-booking-form-state-period-ebd-date"
                type="date"
                placeholder="YYYY-mm-dd"
                required pattern="[0-9]{2}.[0-9]{2}.[0-9]{4}"
                value={this.props.modal.modalEndDate}
                onChange={this.handleEndDateChange}
              />
              <input
                className="form-control"
                id="advanced-booking-form-state-period-end-time"
                type="time"
                placeholder="HH:mm"
                required pattern="[0-9]{2}:[0-9]{2}"
                value={this.props.modal.modalEndTime}
                onChange={this.handleEndTimeChange}
              />
            </div>
          </div>
          <div className={cx(this.props.modal.statusClass)}>
            {this.props.modal.statusMessage}
          </div>
          <button
            type="button"
            className="btn btn-primary"
            onClick={this.handleBookingSave}>
            Save
          </button>
          <button
            type="button"
            className="btn btn-warning"
            onClick={this.props.closeModal}>
            Cancel
          </button>
          <button
            type="button"
            className="btn btn-danger"
            onClick={this.handleBookingDelete}>
            Delete
          </button>
        </form>
      </div>
    );
  }
}

const mapStateToProps = state => {
  const settings = state.settings;
  const modal = state.modal;
  const oldestBooking = state.bookings.oldestBooking;
  return { settings, modal, oldestBooking };
};

export default connect(
  mapStateToProps,
  { handleModalSelectChange, handleStartDateChange, handleStartTimeChange,
  handleEndDateChange, handleEndTimeChange, handleBookingSave,
  handleBookingDelete, closeModal }
)(AdvancedBookingForm);
