import React from 'react';
import ReactDOM from 'react-dom';

import { connect } from "react-redux";
import { saveSettings, getBookings } from "../../redux/advanced/actions";

import BookingBigCalendar from './BookingBigCalendar';
import BookingModal from './BookingModal';

class AdvancedBookingWindow extends React.Component {
  constructor(props) {
    super(props);
    this.props.saveSettings(this.props.dataUnitId, this.props.dataEventType, this.props.dataStates, this.props.dataLanguage, this.props.dataStartTime, this.props.dataEndTime, this.props.dataRefreshInterval, this.props.calendarDefaultView);
  }

  componentDidMount() {
    // Get the list of bookings from server.
    this.getBookings();
    if (this.props.dataRefreshInterval > 0) {
      setInterval(this.getBookings, this.props.dataRefreshInterval);
    }
  }

  getBookings = () => {
    // Fetch the bookings from server and parse them.
    if (this.props.dataStartTimeStore == '' && this.props.dataEndTimeStore == '') {
      this.props.getBookings(this.props.dataUnitId, this.props.dataEventType, this.props.dataStartTime, this.props.dataEndTime, this.props.oldestBooking);
    }
    else {
      this.props.getBookings(this.props.dataUnitId, this.props.dataEventType, this.props.dataStartTimeStore, this.props.dataEndTimeStore, this.props.oldestBooking);
    }
  }

  render() {
    return (
      <div>
        <div>
          <BookingBigCalendar />
        </div>
        <div>
          <BookingModal />
        </div>
      </div>
    );
  }
}

const mapStateToProps = state => {
  const dataStartTimeStore = state.settings.dataStartTime;
  const dataEndTimeStore = state.settings.dataEndTime;
  const oldestBooking = state.bookings.oldestBooking;
  return { dataStartTimeStore, dataEndTimeStore, oldestBooking};
};

export default connect(
  mapStateToProps,
  { saveSettings, getBookings }
)(AdvancedBookingWindow);
