import React from 'react';
import ReactDOM from 'react-dom';

import { connect } from "react-redux";
import {
  editBookingInModal, createBookingInModal, handleCalendarRangeChange
} from "../../redux/advanced/actions";

import BigCalendar from 'react-big-calendar'
import "react-big-calendar/lib/css/react-big-calendar.css";
import "./../../css/big-calendar.css";
import moment from 'moment'
const localizer = BigCalendar.momentLocalizer(moment);

class BookingBigCalendar extends React.Component {
  bookingPropGetter(booking, start, end, isSelected) {
    // If the booking has a color defined, set is background color.
    if (booking.color != null) {
      return {
        style: {
          backgroundColor: booking.color,
        }
      }
    }
  }

  // Delegate the actions to the parent component.
  handleSelectBooking = (booking) => {
    this.props.editBookingInModal(booking);
  }
  handleSelectSlot = ({ start, end }) => {
    this.props.createBookingInModal({ start, end });
  }
  handleRangeChange = (range) => {
    this.props.handleCalendarRangeChange(range, this.props.settings, this.props.bookings.oldestBooking);
  }

  render() {
    return (
      <div>
        <BigCalendar
          selectable
          localizer={localizer}
          events={this.props.bookings.bookings}
          culture={this.props.settings.dataLanguage}
          defaultView={this.props.settings.calendarDefaultView}
          eventPropGetter={this.bookingPropGetter}
          onSelectEvent={this.handleSelectBooking}
          onSelectSlot={this.handleSelectSlot}
          onRangeChange={this.handleRangeChange}
        />
      </div>
    );
  }
}

const mapStateToProps = state => {
  const bookings = state.bookings;
  const settings = state.settings;
  return { bookings, settings };
};

export default connect(
  mapStateToProps,
  { editBookingInModal, createBookingInModal, handleCalendarRangeChange }
)(BookingBigCalendar);
