import React from 'react';
import ReactDOM from 'react-dom';
import { connect } from "react-redux";
import { handleStartDateChange, handleEndDateChange } from "../../redux/simple/actions";

import DatePicker from "react-datepicker";
import "react-datepicker/dist/react-datepicker.css";

class BookingFormDatePicker extends React.Component {
  handleStartChange = (date) => {
    this.props.handleStartDateChange(this.props.dataUnitId, this.props.dataEventType, date, this.props.endDate);
  };
  handleEndChange = (date) => {
    this.props.handleEndDateChange(this.props.dataUnitId, this.props.dataEventType, this.props.startDate, date);
  };

  render() {
    return (
      <div className="booking-date-selector form-group">
        <DatePicker
          selected={this.props.startDate}
          selectsStart
          onChange={this.handleStartChange}
          startDate={this.props.startDate}
          endDate={this.props.endDate}
          dateFormat={drupalSettings.nextpath_simple_booking_settings.startDateFormat}
          showTimeSelect={drupalSettings.nextpath_simple_booking_settings.startShowTimeSelect == 1 ? true : false}
          timeIntervals={parseInt(drupalSettings.nextpath_simple_booking_settings.startTimeIntervals)}
          timeFormat={drupalSettings.nextpath_simple_booking_settings.startTimeFormat}
          disabled={drupalSettings.nextpath_simple_booking_settings.startDateDisable == 1 ? true : false}
          timeCaption={Drupal.t('time')}
        />
        <DatePicker
          selected={this.props.endDate}
          selectsEnd
          onChange={this.handleEndChange}
          startDate={this.props.startDate}
          endDate={this.props.endDate}
          dateFormat={drupalSettings.nextpath_simple_booking_settings.endDateFormat}
          showTimeSelect={drupalSettings.nextpath_simple_booking_settings.endShowTimeSelect == 1 ? true : false}
          showTimeSelectOnly={drupalSettings.nextpath_simple_booking_settings.endShowTimeSelectOnly == 1 ? true : false}
          timeIntervals={parseInt(drupalSettings.nextpath_simple_booking_settings.endTimeIntervals)}
          timeFormat={drupalSettings.nextpath_simple_booking_settings.endTimeFormat}
          disabled={drupalSettings.nextpath_simple_booking_settings.endDateDisable == 1 ? true : false}
          timeCaption={Drupal.t('time')}
        />
      </div>
    );
  }
}

const mapStateToProps = state => {
  const startDate = state.bookingform.startDate;
  const endDate = state.bookingform.endDate;
  const dataUnitId = state.settings.dataUnitId;
  const dataEventType = state.settings.dataEventType;
  return { startDate, endDate, dataUnitId, dataEventType };
};

export default connect(
  mapStateToProps,
  { handleStartDateChange, handleEndDateChange }
)(BookingFormDatePicker);
