import {
  GET_BOOKING_HISTORY_SUCCESS,
  CHECK_AVAILABILITY_STARTED, CHECK_AVAILABILITY_SUCCESS,
  SAVE_SETTINGS,
  SET_START_DATE, SET_END_DATE,
  CREATE_BOOKING_OUTPUT_RESULT,
} from "./actionTypes";

export const handleStartDateChange = (dataUnitId, dataEventType, startDate, endDate) => {
  return dispatch => {
    // Make sure endDate is to before startDate.
    if (startDate >= endDate) {
      let newEndDate = new Date(startDate);
      newEndDate.setHours(newEndDate.getHours() + 1);
      dispatch(setEndDate(newEndDate));
    }
    dispatch(setStartDate(startDate));
    dispatch(checkAvailabilty(dataUnitId, dataEventType, startDate, endDate));
  };
};

export const handleEndDateChange = (dataUnitId, dataEventType, startDate, endDate) => {
  return dispatch => {
    // Make sure endDate is to before startDate.
    if (endDate < startDate) {
      endDate = startDate;
    }

    dispatch(setEndDate(endDate));
    dispatch(checkAvailabilty(dataUnitId, dataEventType, startDate, endDate));
  };
};

const setStartDate = date => ({
  type: SET_START_DATE,
  payload: {
    date
  }
});

const setEndDate = date => ({
  type: SET_END_DATE,
  payload: {
    date
  }
});

export const getBookingHistory = (dataUnitId, dataEventType) => {
  return dispatch => {
    nextpath_client.getBookingHistory(dataUnitId, dataEventType, (bookings) => (
      dispatch(getBookingHistorySuccess(bookings))
    ));
  };
};

const getBookingHistorySuccess = bookings => ({
  type: GET_BOOKING_HISTORY_SUCCESS,
  payload: {
    ...bookings
  }
});

export const checkAvailabilty = (dataUnitId, dataEventType, startDate, endDate) => {
  return dispatch => {
    dispatch(checkAvailabiltyStarted());
    nextpath_client.checkAvailability(dataUnitId, dataEventType, nextpath_helpers.formatDate(startDate), nextpath_helpers.formatDate(endDate), (bookings) => (
      dispatch(checkAvailabiltySuccess(bookings))
    ));
  };
};

const checkAvailabiltyStarted = () => ({
  type: CHECK_AVAILABILITY_STARTED,
  payload: {
    statusMessage: Drupal.t('Looking up for availability...'),
  }
});

const checkAvailabiltySuccess = (bookings) => ({
  type: CHECK_AVAILABILITY_SUCCESS,
  payload: {
    ...bookings
  }
});

export const saveSettings = (dataUnitId, dataEventType, dataEventState) => ({
  type: SAVE_SETTINGS,
  payload: {
    dataUnitId,
    dataEventType,
    dataEventState,
  }
});

export const createBooking = (dataUnitId, dataEventType, dataEventState, startDate, endDate) => {
  return dispatch => {
    const b = nextpath_helpers.createBooking(dataUnitId, dataEventType, dataEventState, nextpath_helpers.formatDate(startDate), nextpath_helpers.formatDate(endDate), null);
    nextpath_client.createBooking(b, dataEventType, (reply) => (
      dispatch(checkCreateBookingResult(reply, dataUnitId, dataEventType))
    ));
  };
};

export const checkCreateBookingResult = (reply, dataUnitId, dataEventType) => {
  return dispatch => {
    if(reply.hasOwnProperty('data') && reply.data.hasOwnProperty('id') ){
      dispatch(createBookingSuccess());
      dispatch(getBookingHistory(dataUnitId, dataEventType));
    }
    else {
      dispatch(createBookingError());
    }
  };
};

export const createBookingSuccess = () => ({
  type: CREATE_BOOKING_OUTPUT_RESULT,
  payload: {
    buttonDisabled: true,
    statusMessage: Drupal.t('Booking was successful'),
    statusClass: 'alert alert-success',
  }
});

export const createBookingError = () => ({
  type: CREATE_BOOKING_OUTPUT_RESULT,
  payload: {
    buttonDisabled: false,
    statusMessage: Drupal.t('An error occured while booking the unit, please try again later'),
    statusClass: 'alert alert-danger',
  }
});
