import moment from 'moment';

window.nextpath_client = (function () {
  function getCsrfToken(callback) {
    return fetch(drupalSettings.path.getHost + drupalSettings.path.baseUrl + '/session/token', {
      method: 'get',
      headers: {
        'Accept': 'application/json',
      },
    }).then(checkStatus)
      .then(parseText)
      .then(callback);
  }

  function checkAvailability(dataUnitId, dataEventType, startDate, endDate, success) {
    var path = 'jsonapi/bat_event/' + dataEventType;

    // Create the conditions groups.
    path += '?filter[and-group-main][group][conjunction]=AND';
    path += '&filter[or-group][group][conjunction]=OR';
    path += '&filter[or-group][group][memberOf]=and-group-main';
    path += '&filter[and-group-sub1][group][conjunction]=AND';
    path += '&filter[and-group-sub1][group][memberOf]=or-group';
    path += '&filter[and-group-sub2][group][conjunction]=AND';
    path += '&filter[and-group-sub2][group][memberOf]=or-group';
    path += '&filter[and-group-sub3][group][conjunction]=AND';
    path += '&filter[and-group-sub3][group][memberOf]=or-group';

    // Add the unit reference condition
    path += '&filter[unitId][condition][path]=event_bat_unit_reference.id';
    path += '&filter[unitId][condition][operator]=%3D'; // =
    path += '&filter[unitId][condition][value]=' + dataUnitId;
    path += '&filter[unitId][condition][memberOf]=and-group-main';

    // Add the blocking state condition
    path += '&filter[state][condition][path]=event_state_reference.blocking';
    path += '&filter[state][condition][operator]=%3D'; // =
    path += '&filter[state][condition][value]=1';
    path += '&filter[state][condition][memberOf]=and-group-main';

    // TTB = time to be booked.
    // Check for events that start before TTB_start_date and ends after
    // TTB_start_date.
    path += '&filter[date_1_1][condition][path]=event_dates.value';
    path += '&filter[date_1_1][condition][operator]=%3C%3D'; // <=
    path += '&filter[date_1_1][condition][value]=' + startDate;
    path += '&filter[date_1_1][condition][memberOf]=and-group-sub1';
    path += '&filter[date_1_2][condition][path]=event_dates.end_value';
    path += '&filter[date_1_2][condition][operator]=%3E'; // >
    path += '&filter[date_1_2][condition][value]=' + startDate;
    path += '&filter[date_1_2][condition][memberOf]=and-group-sub1';

    // Check for events that start before TTB_end_date and ends after
    // TTB_end_date.
    path += '&filter[date_2_1][condition][path]=event_dates.value';
    path += '&filter[date_2_1][condition][operator]=%3C'; // <
    path += '&filter[date_2_1][condition][value]=' + endDate;
    path += '&filter[date_2_1][condition][memberOf]=and-group-sub2';
    path += '&filter[date_2_2][condition][path]=event_dates.end_value';
    path += '&filter[date_2_2][condition][operator]=%3E%3D'; // >=
    path += '&filter[date_2_2][condition][value]=' + endDate;
    path += '&filter[date_2_2][condition][memberOf]=and-group-sub2';

    // Check for events that start after TTB_start_date and ends before
    // TTB_end_date.
    path += '&filter[date_3_1][condition][path]=event_dates.value';
    path += '&filter[date_3_1][condition][operator]=%3E%3D'; // >=
    path += '&filter[date_3_1][condition][value]=' + startDate;
    path += '&filter[date_3_1][condition][memberOf]=and-group-sub3';
    path += '&filter[date_3_2][condition][path]=event_dates.end_value';
    path += '&filter[date_3_2][condition][operator]=%3C%3D'; // <=
    path += '&filter[date_3_2][condition][value]=' + endDate;
    path += '&filter[date_3_2][condition][memberOf]=and-group-sub3';

    // Add limit.
    path += '&page[limit]=1';

    return fetch(drupalSettings.path.getHost + drupalSettings.path.baseUrl + path, {
      headers: {
        Accept: 'application/vnd.api+json',
      },
    }).then(checkStatus)
      .then(parseJSON)
      .then(success);
  }

  function createBooking(data, dataEventType, success) {
    var url = 'jsonapi/bat_event/' + dataEventType;
    var fetch_method = 'POST';
    if (data.data.hasOwnProperty('id')) {
      url += '/' + data.data.id;
      fetch_method = 'PATCH';
      url += '?include=event_state_reference';
    }

    getCsrfToken(function (csrfToken) {
      return fetch(drupalSettings.path.getHost + drupalSettings.path.baseUrl + url, {
        method: fetch_method,
        body: JSON.stringify(data),
        headers: {
          'Accept': 'application/vnd.api+json',
          'Content-Type': 'application/vnd.api+json',
          'X-CSRF-Token': csrfToken,
        },
      }).then(checkStatus)
        .then(parseJSON)
        .then(success);
    });
  }

  function deleteBooking(id, dataEventType, success) {
    var url = 'jsonapi/bat_event/' + dataEventType + '/' + id;

    getCsrfToken(function (csrfToken) {
      return fetch(drupalSettings.path.getHost + drupalSettings.path.baseUrl + url, {
        method: 'DELETE',
        headers: {
          'Accept': 'application/vnd.api+json',
          'Content-Type': 'application/vnd.api+json',
          'X-CSRF-Token': csrfToken,
        },
      }).then(checkStatus)
        .then(success);
    });
  }

  function getBookingHistory(dataUnitId, dataEventType, success) {
    var path = 'jsonapi/bat_event/' + dataEventType;

    // Add the unit reference condition.
    path += '?filter[unitId][condition][path]=event_bat_unit_reference.id';
    path += '&filter[unitId][condition][operator]=%3D'; // =
    path += '&filter[unitId][condition][value]=' + dataUnitId;
    // Add the author condition.
    path += '&filter[uid][condition][path]=uid.id';
    path += '&filter[uid][condition][operator]=%3D'; // =
    path += '&filter[uid][condition][value]=' + drupalSettings.user.uuid;
    // Add sorting and limit.
    path += '&sort[sort-created][path]=created';
    path += '&sort[sort-created][direction]=DESC';
    path += '&page[limit]=5';

    return fetch(drupalSettings.path.getHost + drupalSettings.path.baseUrl + path, {
      headers: {
        Accept: 'application/vnd.api+json',
      },
    }).then(checkStatus)
      .then(parseJSON)
      .then(success);
  }

  function getBookings(dataUnitId, dataEventType, startTime, endTime, oldestBooking, success) {
    var path = 'jsonapi/bat_event/' + dataEventType;

    // Add the unit reference condition.
    path += '?filter[unitId][condition][path]=event_bat_unit_reference.id';
    path += '&filter[unitId][condition][operator]=%3D'; // =
    path += '&filter[unitId][condition][value]=' + dataUnitId;
    // Add sorting and limit.
    path += '&sort[sort-created][path]=changed';
    path += '&sort[sort-created][direction]=ASC';

    // Add the event dates conditions.
    path += '&filter[date_1][condition][path]=event_dates.value';
    path += '&filter[date_1][condition][operator]=%3E%3D'; // >=
    path += '&filter[date_1][condition][value]=' + startTime;
    path += '&filter[date_2][condition][path]=event_dates.end_value';
    path += '&filter[date_2][condition][operator]=%3C%3D'; // <=
    path += '&filter[date_2][condition][value]=' + endTime;

    if (oldestBooking != null) {
      // Add the changed condition.
      path += '&filter[changed][condition][path]=changed';
      path += '&filter[changed][condition][operator]=%3E'; // >
      path += '&filter[changed][condition][value]=' + oldestBooking;
    }

    // Add the details of state.
    path += '&include=event_state_reference';

    return fetch(drupalSettings.path.getHost + drupalSettings.path.baseUrl + path, {
      headers: {
        Accept: 'application/vnd.api+json',
      },
    }).then(checkStatus)
      .then(parseJSON)
      .then(success);
  }

  function checkStatus(response) {
    if (response.status >= 200 && response.status < 300) {
      return response;
    } else {
      const error = new Error(`HTTP Error ${response.statusText}`);
      error.status = response.statusText;
      error.response = response;
      console.log(error);
      throw error;
    }
  }

  function parseJSON(response) {
    return response.json();
  }

  function parseText(response) {
    return response.text();
  }

  function reverseArray(response) {
    return response.reverse();
  }

  return {
    checkAvailability,
    createBooking,
    getBookingHistory,
    getBookings,
    deleteBooking,
  };
}());

window.nextpath_helpers = (function () {
  function pad(n, width, z) {
    z = z || '0';
    n = n + '';
    return n.length >= width ? n : new Array(width - n.length + 1).join(z) + n;
  }

  function formatDate(date) {
    var year = date.getFullYear();
    var month = date.getMonth() + 1;
    var day = date.getDate();
    var hours = date.getHours();
    var minutes = date.getMinutes();
    //var seconds = date.getSeconds();
    var seconds = 0;
    return pad(year, 4) + '-' + pad(month, 2) + '-' + pad(day, 2) + 'T' + pad(hours, 2) + ':' + pad(minutes, 2) + ':' + pad(seconds, 2);
  }

  function formatOutputDate(old_date, utc) {
    if (utc == true) {
      return moment.utc(old_date).format('DD.MM.YYYY HH:mm');
    }
    else {
      return moment(old_date).format('DD.MM.YYYY HH:mm');
    }
  }

  function increaseDate(date, days) {
    var newDate = new Date(date);
    newDate.setDate(newDate.getDate() + days);
    return newDate;
  }

  function removeTimezone(old_date) {
    var new_date =  old_date.substring(0,19);
    return new_date;
  }

  function createBooking(dataUnitId, dataEventType, dataEventState, startDate, endDate, dataEventId) {
    var booking = {
      data: {
        type: 'bat_event--' + dataEventType,
        attributes: {
          event_dates: {
            value: startDate,
            end_value: endDate,
          },
        },
        relationships: {
          event_bat_unit_reference: {
            data: {
              type: 'bat_unit--default',
              id: dataUnitId,
            },
          },
          event_state_reference: {
            data: {
              type: 'state--state',
              id: dataEventState,
            },
          },
        },
      },
    };
    if (dataEventId != null) {
      booking.data.id = dataEventId;
    }
    return booking;
  }

  function getStatesFromRelation(bookings) {
    var states = {};
    if (bookings.hasOwnProperty('included')) {
      Array.prototype.forEach.call(bookings.included, function(element) {
        states[element.id] = {
          name: element.attributes.name,
          color: element.attributes.color,
        }
      });
    }
    return states
  }

  function parseBookings(bookings) {
    // Get the states related to the bookings received.
    var states = getStatesFromRelation(bookings);

    var newBookings = [];
    Array.prototype.forEach.call(bookings.data, function(element) {
      var title = element.id;
      var color = null;
      var stateId = null;
      if (states.hasOwnProperty(element.relationships.event_state_reference.data.id)) {
        title = states[element.relationships.event_state_reference.data.id].name;
        color = states[element.relationships.event_state_reference.data.id].color;
        stateId = element.relationships.event_state_reference.data.id;
      }
      newBookings.push({
        title: title,
        start: new Date(removeTimezone(element.attributes.event_dates.value)),
        end: new Date(removeTimezone(element.attributes.event_dates.end_value)),
        id: element.id,
        created: element.attributes.created,
        changed: element.attributes.changed,
        color: color,
        stateId: stateId,
      });
    });

    return newBookings;
  }

  function validateBooking(modalStartDate, modalStartTime, modalEndDate, modalEndTime, modalStateSelect) {
    var error = [];
    if (modalStartDate == '') {
      error.push(Drupal.t('Start date is mandatory'));
    }
    if (modalStartTime == '') {
      error.push(Drupal.t('Start time is mandatory'));
    }
    if (modalEndDate == '') {
      error.push(Drupal.t('End date is mandatory'));
    }
    if (modalEndTime == '') {
      error.push(Drupal.t('End time is mandatory'));
    }
    if (modalEndTime == '') {
      error.push(Drupal.t('End time is mandatory'));
    }
    if (modalStateSelect == '') {
      error.push(Drupal.t('State is mandatory'));
    }
    var startDate = new Date(modalStartDate + ' ' + modalStartTime);
    var endDate = new Date(modalEndDate + ' ' + modalEndTime);
    if (startDate >= endDate) {
      error.push(Drupal.t('Start date should be before end date'));
    }
    return error;
  }

  function removeDuplicateBookings(bookings) {
    // Search for duplicates and eliminate them.
    // In case of duplicates, keep the one high higher changed date.
    for (var i = 0 ; i < bookings.length; i++) {
      for (var j = i + 1; j < bookings.length; j++) {
        if (bookings[i].id === bookings[j].id) {
          if (bookings[i].changed >= bookings[j].changed) {
            bookings.splice(j, 1);
            j--;
          }
          else {
            bookings.splice(i, 1);
            i--;
          }
        }
      }
    }
    return bookings;
  }

  return {
    formatDate,
    pad,
    createBooking,
    formatOutputDate,
    parseBookings,
    validateBooking,
    removeTimezone,
    getStatesFromRelation,
    increaseDate,
    removeDuplicateBookings,
  };
}());
